@@ -12,7 +12,7 @@ __all__ = (
     'EUI48', 'EUI60', 'EUI64',
     'parse',
 )
-__version__ = '1.0.1'
+__version__ = '1.1.0'
 
 
 _HEX_DIGITS = "0123456789ABCDEFabcdef"
@@ -22,17 +22,26 @@ def _name(obj):
     return type(obj).__name__
 
 
-def _value_error(value, error, *classes):
+def _class_names_in_proper_english(classes):
     class_names = [cls.__name__ for cls in classes]
     number_of_classes = len(classes)
     if number_of_classes < 2:
-        class_description = class_names[0]
+        return class_names[0]
     elif number_of_classes == 2:
-        class_description = ' or '.join(class_names)
+        return ' or '.join(class_names)
     else:
         class_names[-1] = 'or ' + class_names[-1]
-        class_description = ', '.join(class_names)
-    return ValueError(repr(value) + ' ' + error + ' ' + class_description)
+        return ', '.join(class_names)
+
+
+def _type_error(value, *classes):
+    class_names = _class_names_in_proper_english(classes)
+    return TypeError(repr(value) + ' has wrong type for ' + class_names)
+
+
+def _value_error(value, error, *classes):
+    class_names = _class_names_in_proper_english(classes)
+    return ValueError(repr(value) + ' ' + error + ' ' + class_names)
 
 
 class HWAddress:
@@ -60,10 +69,12 @@ class HWAddress:
 
         Arguments:
             address: An ``int``, ``bytes``, or ``str`` representation
-                of the address. If a string, it is parsed using the
-                ``formats`` attribute of the class. If a byte string,
-                it is read in big-endian. If it is an integer, bytes
-                in the integer value are used as the address bytes.
+                of the address, or another instance of the same class
+                of address. If a string, the ``formats`` attribute of
+                the class is used to parse it. If a byte string, it
+                is read in big-endian. If an integer, its value bytes
+                in big-endian are used as the address bytes. If an
+                instance of the same address class, its value is used.
 
         Raises:
             TypeError: If ``address`` is not one of the valid types.
@@ -88,8 +99,10 @@ class HWAddress:
             self._address = int.from_bytes(address, 'big') >> offset
         elif isinstance(address, str):
             self._address, _ = _parse(address, type(self))
+        elif isinstance(address, type(self)):
+            self._address = address._address
         else:
-            raise TypeError(_name(self) + ' cannot parse ' + _name(address))
+            raise _type_error(address, type(self))
 
     def __repr__(self):
         """Represent the hardware address as an unambiguous string."""
@@ -299,11 +312,9 @@ class EUI64(_StartsWithOUI):
     )
 
 
-def parse(string, *classes):
-    """Try parsing a string as several hardware address classes at once.
+def parse(value, *classes):
+    """Try parsing a value as several hardware address classes at once.
 
-    This is useful when you have one piece of code that can accept user
-    input of two or more different hardware addresses or identifiers.
     This lets you can just write
 
         address = hwaddress.parse(user_input, EUI64, EUI48, ...)
@@ -311,32 +322,46 @@ def parse(string, *classes):
     instead of all of this:
 
         try:
-            hw_address = hwaddress.EUI64(user_input)
+            address = hwaddress.EUI64(user_input)
         except ValueError:
             try:
-                hw_address = hwaddress.EUI48(user_input)
+                address = hwaddress.EUI48(user_input)
             except ValueError:
                 ...
 
     Arguments:
-        string: The string to parse as a hardware address.
+        value: The value to parse as a hardware address. Either a
+            string, byte string, or an instance of one of the classes.
         *classes: HWAddress subclasses to try to parse the string as.
+            If the input address could parse as more than one of the
+            classes, it is parsed as the first one.
 
     Returns:
-        HWAddress: The parsed hardware address.
-        None: If no classes were passed in.
+        HWAddress: The parsed hardware address if the value argument
+            was a string or byte string, or the value argument itself
+            if it was already an instance of one of the classes.
 
     Raises:
-        ValueError: If the string could not be parsed as any of the
-            given classes.
-        TypeError: If the string is not actually a string.
+        TypeError: If the value is not one of the valid types,
+            or if no classes were passed in.
+        ValueError: If the value could not be parsed as any
+            of the given classes.
     """
-    if not isinstance(string, str):
-        return TypeError
     if not classes:
-        return None
-    address, cls = _parse(string, *classes)
-    return cls(address)
+        raise TypeError('parse() requires at least one class argument')
+    if isinstance(value, str):
+        address, cls = _parse(value, *classes)
+        return cls(address)
+    elif isinstance(value, bytes):
+        max_size = len(value) * 8
+        min_size = max_size - 7
+        for cls in classes:
+            if min_size <= cls.size <= max_size:
+                return cls(value)
+        raise _value_error(value, 'has wrong length for', *classes)
+    elif isinstance(value, classes):
+        return value
+    raise _type_error(value, *classes)
 
 
 def _parse(string, *classes):
